package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/lib/comm_plan"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"context"
	"errors"
	"golang.org/x/sync/errgroup"
	"sync"
	"xorm.io/xorm"
)

// BatchGetCommission 批量分佣
func BatchGetCommission(eg *xorm.Engine, dbName string, commissionParamList []*md.CommissionFirstParam) (map[string]*comm_plan.LvUser, error) {
	var (
		isShare          = false
		isAllLevelReturn = false
	)
	goodsId2lvUser := make(map[string]*comm_plan.LvUser, len(commissionParamList))
	if len(commissionParamList) == 0 {
		return goodsId2lvUser, errors.New("参数错误")
	}

	group, _ := errgroup.WithContext(context.Background())
	var mu sync.Mutex

	for _, param := range commissionParamList {
		param := param // 为下面的闭包创建局部变量
		group.Go(func() error {
			if param.IsShare != 0 {
				isShare = true
			}

			if param.IsAllLevelReturn != 0 {
				isAllLevelReturn = true
			}

			if param.GoodsId == "" {
				return errors.New("商品ID缺失")
			}

			_, _, _, _, lvUser, err := GetRewardCommission(eg, &param.CommissionParam, isShare, param.Uid, param.Provider, dbName, isAllLevelReturn, map[string]string{})
			if err != nil {
				return err
			}
			mu.Lock()
			goodsId2lvUser[param.GoodsId] = lvUser
			mu.Unlock()

			return nil
		})
	}
	if err := group.Wait(); err != nil {
		return goodsId2lvUser, errors.New("处理错误")
	}

	return goodsId2lvUser, nil
}
